@@ -26,7 +26,9 @@ from . import defdict
 from . import nodes
 from . import generators
 from . import comment
+from . import example
 from . import documentmerger
+from . import utf8
 
 def nsgtk(s):
     return '{{{0}}}{1}'.format('http://www.gtk.org/introspection/core/1.0', s)
@@ -113,6 +115,7 @@ class GirComment(comment.Comment):
     refsect2 = re.compile('(<refsect2 [^>]*>|</refsect2>)\n?', re.I)
     varref = re.compile('@([a-z][a-z0-9_]*)', re.I)
     constref = re.compile('%([a-z_][a-z0-9_]*)', re.I)
+    proglisting = re.compile('<informalexample>\s*<programlisting>\s*(.*?)\s*</programlisting>\s*</informalexample>', re.I | re.M)
 
     def __init__(self, cursor):
         doc = cursor.node.find(nsgtk('doc'))
@@ -183,7 +186,7 @@ class GirComment(comment.Comment):
                     preat.append('')
 
             if brieftext == '':
-                brieftext = '*Documentation missing...*'
+                brieftext = '*documentation missing...*'
 
             text = brieftext.replace('\n', ' ').rstrip() + "\n" + "\n".join(preat) + doctext
 
@@ -204,6 +207,7 @@ class GirComment(comment.Comment):
         text = GirComment.emph.sub(lambda x: '*{0}*'.format(x.group(1)), text)
         text = GirComment.title.sub(lambda x: '## {0}'.format(x.group(1)), text)
         text = GirComment.refsect2.sub(lambda x: '', text)
+        text = GirComment.proglisting.sub(lambda x: '    [code]\n    {0}\n'.format(x.group(1).replace('\n', '\n    ')), text)
 
         return text
 
@@ -355,6 +359,11 @@ class GirCursor:
         self.bases = None
         self.implements = None
 
+        if 'introspectable' in node.attrib:
+            self.introspectable = (node.attrib['introspectable'] != '0')
+        else:
+            self.introspectable = True
+
         self.type = self._extract_type()
         self.kind = self._extract_kind()
 
@@ -537,6 +546,14 @@ class GirCursor:
         else:
             return None
 
+    @property
+    def extent(self):
+        return None
+
+    @property
+    def location(self):
+        return None
+
     def get_children(self):
         return self.children
 
@@ -614,7 +631,7 @@ class GirTree(documentmerger.DocumentMerger):
             self.root_node = self.root
 
     def match_ref(self, child, name):
-        if isinstance(name, basestring):
+        if isinstance(name, utf8.string):
             return name == child.name
         else:
             return name.match(child.name)
@@ -638,12 +655,14 @@ class GirTree(documentmerger.DocumentMerger):
         if node is None:
             node = self.root
 
-        if node.comment:
+        if not node.comment is None:
             node.comment.resolve_refs(self.find_ref, node)
 
         for child in node.children:
             self.cross_ref(child)
 
+        self.markup_code()
+
     def parse_function(self, cursor):
         return nodes.Function(cursor, GirComment(cursor))
 
@@ -730,6 +749,9 @@ class GirTree(documentmerger.DocumentMerger):
         pass
 
     def parse_cursor(self, cursor):
+        if not cursor.introspectable:
+            return None
+
         fn = self.mapping[cursor.typename]
 
         if not fn is None:
@@ -848,6 +870,30 @@ class GirTree(documentmerger.DocumentMerger):
         for node in self.all_nodes:
             self.qid_to_node[node.qid] = node
 
+    def markup_code(self):
+        for node in self.all_nodes:
+            if node.comment is None:
+                continue
+
+            if not node.comment.doc:
+                continue
+
+            comps = node.comment.doc.components
+
+
+            for i in range(len(comps)):
+                component = comps[i]
+
+                if not isinstance(component, comment.Comment.Example):
+                    continue
+
+                text = str(component)
+
+                ex = example.Example()
+                ex.append(text)
+
+                comps[i] = ex
+
 def run(args):
     parser = argparse.ArgumentParser(description='clang based documentation generator.',
                                      usage='%(prog)s gir --output DIR [OPTIONS] GIRFILE')
