@@ -12,18 +12,12 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 from clang import cindex
 from defdict import Defdict
+
 from cldoc.struct import Struct
+from cldoc import utf8
 
 import os, re, sys, bisect
 
-def make_unicode(s):
-    if isinstance(s, unicode):
-        return s
-    elif isinstance(s, basestring):
-        return unicode(s, 'utf-8')
-    else:
-        return make_unicode(str(s))
-
 class Sorted(list):
     def __init__(self, key=None):
         if key is None:
@@ -83,28 +77,51 @@ class Comment(object):
 
     class String(object):
         def __init__(self, s):
-            self.components = [s]
+            self.components = [utf8.utf8(s)]
 
-        def __unicode__(self):
-            return u"".join(self.components)
+        def _utf8(self):
+            return utf8.utf8("").join([utf8.utf8(x) for x in self.components])
 
         def __str__(self):
-            return unicode(self).encode('utf-8')
+            return str(self._utf8())
+
+        def __unicode__(self):
+            return unicode(self._utf8())
+
+        def __bytes__(self):
+            return bytes(self._utf8())
+
+        def __eq__(self, other):
+            if isinstance(other, str):
+                return str(self) == other
+            elif isinstance(other, unicode):
+                return unicode(self) == other
+            elif isinstance(other, bytes):
+                return bytes(self) == other
+            else:
+                return object.__cmp__(self, other)
 
         def __nonzero__(self):
             l = len(self.components)
 
             return l > 0 and (l > 1 or len(self.components[0]) > 0)
 
-    class UnresolvedReference(str):
+    class MarkdownCode(utf8.utf8):
+        pass
+
+    class UnresolvedReference(utf8.utf8):
         reescape = re.compile('[*_]', re.I)
 
-        def __new__(self, s):
-            s = Comment.UnresolvedReference.reescape.sub(lambda x: '\\' + x.group(0), s)
-            return str.__new__(self, '<' + s + '>')
+        def __new__(cls, s):
+            ns = Comment.UnresolvedReference.reescape.sub(lambda x: '\\' + x.group(0), s)
+            ret = utf8.utf8.__new__(cls, utf8.utf8('&lt;{0}&gt;').format(utf8.utf8(ns)))
 
-    redocref = re.compile('(?P<isregex>[$]?)<(?:\\[(?P<refname>[^\\]]*)\\])?(?P<ref>operator(?:>>|>|>=)|[^>]+)>')
+            ret.orig = s
+            return ret
+
+    redocref = re.compile('(?P<isregex>[$]?)<(?:\\[(?P<refname>[^\\]]*)\\])?(?P<ref>operator(?:>>|>|>=)|[^>\n]+)>')
     redoccode = re.compile('^    \\[code\\]\n(?P<code>(?:(?:    .*|)\n)*)', re.M)
+    redocmcode = re.compile('(^ *(`{3,}|~{3,}).*?\\2)', re.M | re.S)
 
     def __init__(self, text, location):
         self.__dict__['docstrings'] = []
@@ -123,47 +140,69 @@ class Comment(object):
         if isinstance(val, dict):
             for key in val:
                 if not isinstance(val[key], Comment.String):
-                    val[key] = Comment.String(make_unicode(val[key]))
+                    val[key] = Comment.String(val[key])
         elif not isinstance(val, Comment.String):
-            val = Comment.String(make_unicode(val))
+            val = Comment.String(val)
 
         self.__dict__[name] = val
 
+    def __nonzero__(self):
+        return (bool(self.brief) and not (self.brief == u'*documentation missing...*')) or (bool(self.doc) and not (self.doc == u'*documentation missing...*'))
+
+    def redoccode_split(self, doc):
+        # Split on C/C++ code
+        components = Comment.redoccode.split(doc)
+        ret = []
+
+        for i in range(0, len(components), 2):
+            r = Comment.redocmcode.split(components[i])
+
+            for j in range(0, len(r), 3):
+                ret.append(r[j])
+
+                if j < len(r) - 1:
+                    ret.append(Comment.MarkdownCode(r[j + 1]))
+
+            if i < len(components) - 1:
+                ret.append(Comment.Example(components[i + 1]))
+
+        return ret
+
     def redoc_split(self, doc):
         ret = []
 
         # First split examples
-        components = Comment.redoccode.split(doc)
-        for i in range(0, len(components), 2):
-            rdoc = components[i]
-            lastpos = 0
+        components = self.redoccode_split(doc)
 
-            for m in Comment.redocref.finditer(rdoc):
-                span = m.span(0)
+        for c in components:
+            if isinstance(c, Comment.Example) or isinstance(c, Comment.MarkdownCode):
+                ret.append((c, None, None))
+            else:
+                lastpos = 0
 
-                prefix = rdoc[lastpos:span[0]]
-                lastpos = span[1]
+                for m in Comment.redocref.finditer(c):
+                    span = m.span(0)
 
-                ref = m.group('ref')
-                refname = m.group('refname')
+                    prefix = c[lastpos:span[0]]
+                    lastpos = span[1]
 
-                if not refname:
-                    refname = None
+                    ref = m.group('ref')
+                    refname = m.group('refname')
 
-                if len(m.group('isregex')) > 0:
-                    ref = re.compile(ref)
+                    if not refname:
+                        refname = None
 
-                ret.append((prefix, ref, refname))
+                    if len(m.group('isregex')) > 0:
+                        ref = re.compile(ref)
 
-            ret.append((rdoc[lastpos:], None, None))
+                    ret.append((prefix, ref, refname))
 
-            if i < len(components) - 1:
-                ret.append((Comment.Example(components[i + 1]), None, None))
+                ret.append((c[lastpos:], None, None))
 
         return ret
 
     def resolve_refs_for_doc(self, doc, resolver, root):
-        comps = self.redoc_split(make_unicode(doc))
+        comps = self.redoc_split(utf8.utf8(doc))
         components = []
 
         for pair in comps:
@@ -173,7 +212,7 @@ class Comment(object):
             if name is None:
                 continue
 
-            if isinstance(name, basestring):
+            if isinstance(name, utf8.string):
                 names = name.split('::')
             else:
                 names = [name]
@@ -213,7 +252,7 @@ class Comment(object):
             if isinstance(doc, dict):
                 for key in doc:
                     if not isinstance(doc[key], Comment.String):
-                        doc[key] = Comment.String(make_unicode(doc[key]))
+                        doc[key] = Comment.String(doc[key])
 
                     self.resolve_refs_for_doc(doc[key], resolver, root)
             else:
@@ -374,7 +413,7 @@ class CommentsDatabase(object):
             cleaned = self.clean(token)
 
             # Process instructions directly, now
-            if not CommentsDatabase.cldoc_instrre.match(cleaned) is None:
+            if (not cleaned is None) and (not CommentsDatabase.cldoc_instrre.match(cleaned) is None):
                 comments = [cleaned]
                 break
 
