@@ -16,6 +16,7 @@ from cldoc.clang import cindex
 from .generator import Generator
 from cldoc import nodes
 from cldoc import example
+from cldoc import utf8
 
 from xml.etree import ElementTree
 import sys, os
@@ -227,12 +228,24 @@ class Xml(Generator):
         if not node.typedef is None:
             elem.set('typedef', 'yes')
 
+    def templatetypeparameter_to_xml(self, node, elem):
+        dt = node.default_type
+
+        if not dt is None:
+            d = ElementTree.Element('default')
+
+            d.append(self.type_to_xml(dt))
+            elem.append(d)
+
+    def templatenontypeparameter_to_xml(self, node, elem):
+        elem.append(self.type_to_xml(node.type))
+
     def function_to_xml(self, node, elem):
         if not (isinstance(node, nodes.Constructor) or
                 isinstance(node, nodes.Destructor)):
             ret = ElementTree.Element('return')
 
-            if node.comment and hasattr(node.comment, 'returns') and node.comment.returns:
+            if not node.comment is None and hasattr(node.comment, 'returns') and node.comment.returns:
                 ret.append(self.doc_to_xml(node, node.comment.returns))
 
             tp = self.type_to_xml(node.return_type, node.parent)
@@ -245,7 +258,7 @@ class Xml(Generator):
             ret.set('name', arg.name)
             ret.set('id', arg.qid)
 
-            if node.comment and arg.name in node.comment.params:
+            if not node.comment is None and arg.name in node.comment.params:
                 ret.append(self.doc_to_xml(node, node.comment.params[arg.name]))
 
             ret.append(self.type_to_xml(arg.type, node.parent))
@@ -302,7 +315,7 @@ class Xml(Generator):
 
             child.append(self.type_to_xml(base.type, node))
 
-            if base.node and base.node.comment and base.node.comment.brief:
+            if base.node and not base.node.comment is None and base.node.comment.brief:
                 child.append(self.doc_to_xml(base.node, base.node.comment.brief, 'brief'))
 
             elem.append(child)
@@ -316,7 +329,7 @@ class Xml(Generator):
 
             child.set('name', subcls.qid_to(node.qid))
 
-            if subcls.comment and subcls.comment.brief:
+            if not subcls.comment is None and subcls.comment.brief:
                 child.append(self.doc_to_xml(subcls, subcls.comment.brief, 'brief'))
 
             elem.append(child)
@@ -353,7 +366,7 @@ class Xml(Generator):
         last = None
 
         for component in doc.components:
-            if isinstance(component, basestring):
+            if isinstance(component, utf8.string):
                 s += component
             elif isinstance(component, example.Example):
                 # Make highlighting
@@ -387,7 +400,10 @@ class Xml(Generator):
                         par.text = item.text
                         last = par
 
-                last.tail = s
+                if last == code:
+                    last.text = s
+                else:
+                    last.tail = s
 
                 s = ''
                 last = code
@@ -411,7 +427,7 @@ class Xml(Generator):
                     if refname:
                         last.text = refname
                     else:
-                        last.text = parent.qid_from(cc.qid)
+                        last.text = parent.qlbl_from(cc)
 
                     self.add_ref_node_id(cc, last)
 
@@ -431,16 +447,23 @@ class Xml(Generator):
         return doce
 
     def call_type_specific(self, node, elem, fn):
-        cls = node.__class__
+        clss = [node.__class__]
+
+        while len(clss) > 0:
+            cls = clss[0]
+            clss = clss[1:]
+
+            if cls == nodes.Node:
+                continue
 
-        while cls != nodes.Node:
             nm = cls.__name__.lower() + '_' + fn
 
             if hasattr(self, nm):
                 getattr(self, nm)(node, elem)
                 break
 
-            cls = cls.__base__
+            if cls != nodes.Node:
+                clss.extend(cls.__bases__)
 
     def node_to_xml(self, node):
         elem = ElementTree.Element(node.classname)
@@ -450,10 +473,10 @@ class Xml(Generator):
             if props[prop]:
                 elem.set(prop, props[prop])
 
-        if node.comment and node.comment.brief:
+        if not node.comment is None and node.comment.brief:
             elem.append(self.doc_to_xml(node, node.comment.brief, 'brief'))
 
-        if node.comment and node.comment.doc:
+        if not node.comment is None and node.comment.doc:
             elem.append(self.doc_to_xml(node, node.comment.doc))
 
         self.call_type_specific(node, elem, 'to_xml')
@@ -473,6 +496,13 @@ class Xml(Generator):
 
         return elem
 
+    def templated_to_xml_ref(self, node, element):
+        for child in node.sorted_children():
+            if not (isinstance(child, nodes.TemplateTypeParameter) or isinstance(child, nodes.TemplateNonTypeParameter)):
+                continue
+
+            element.append(self.node_to_xml(child))
+
     def generate_page(self, node):
         elem = self.node_to_xml(node)
         self.write_xml(elem, node.qid + '.xml')
@@ -487,7 +517,7 @@ class Xml(Generator):
         if 'name' in props:
             elem.set('name', props['name'])
 
-        if node.comment and node.comment.brief:
+        if not node.comment is None and node.comment.brief:
             elem.append(self.doc_to_xml(node, node.comment.brief, 'brief'))
 
         self.call_type_specific(node, elem, 'to_xml_ref')
