@@ -22,4 +22,11 @@ class Category(Node):
     def name(self):
         return self._name
 
+    @property
+    def is_unlabeled(self):
+        return True
+
+    def sorted_children(self):
+        return list(self.children)
+
 # vi:ts=4:et
