@@ -19,7 +19,7 @@ class Field(Node):
 
     def __init__(self, cursor, comment):
         Node.__init__(self, cursor, comment)
-        self.type = Type(cursor.type)
+        self.type = Type(cursor.type, cursor=cursor)
 
     def compare_same(self, other):
         return cmp(self.sort_index, other.sort_index)
