@@ -22,8 +22,16 @@ class Argument:
     def __init__(self, func, cursor):
         self.cursor = cursor
         self.parent = func
+        self._type = None
+
+        for child in cursor.get_children():
+            if child.kind == cindex.CursorKind.TYPE_REF:
+                self._type = Type(self.cursor.type, cursor=child)
+                break
+
+        if self._type is None:
+            self._type = Type(self.cursor.type)
 
-        self._type = Type(self.cursor.type)
         self._refid = None
 
     @property
@@ -46,14 +54,33 @@ class Argument:
     def force_page(self):
         return False
 
+    def semantic_path_until(self, other):
+        ret = self.parent.semantic_path_until(other)
+        ret.append(self)
+
+        return ret
+
+    def qlbl_from(self, other):
+        return self.parent.qlbl_from(other) + '::' + self.name
+
+    def qlbl_to(self, other):
+        return other.qlbl_from(self)
+
+    @property
+    def semantic_parent(self):
+        return self.parent
+
+    @property
+    def is_unlabeled(self):
+        return False
+
 class Function(Node):
     kind = cindex.CursorKind.FUNCTION_DECL
 
     def __init__(self, cursor, comment):
-        Node.__init__(self, cursor, comment)
+        super(Function, self).__init__(cursor, comment)
 
         self._return_type = Type(self.cursor.type.get_result())
-
         self._arguments = []
 
         for child in cursor.get_children():
@@ -66,6 +93,15 @@ class Function(Node):
     def qid(self):
         return self.name
 
+    @property
+    def semantic_parent(self):
+        from namespace import Namespace
+
+        if isinstance(self.parent, Namespace):
+            return self.parent
+        else:
+            return None
+
     @property
     def resolve_nodes(self):
         for arg in self._arguments:
@@ -77,19 +113,15 @@ class Function(Node):
             yield k.name
 
     def parse_comment(self):
-        m = Parser.parse(self._comment.text)
+        super(Function, self).parse_comment()
         self._comment.params = {}
 
-        if len(m.brief) > 0:
-            self._comment.brief = m.brief
-            self._comment.doc = m.body
+        for pre in self._parsed_comment.preparam:
+            self._comment.params[pre.name] = pre.description
 
-            for pre in m.preparam:
-                self._comment.params[pre.name] = pre.description
-
-            for post in m.postparam:
-                if post.name == 'return':
-                    self._comment.returns = post.description
+        for post in self._parsed_comment.postparam:
+            if post.name == 'return':
+                self._comment.returns = post.description
 
     @property
     def return_type(self):
