@@ -13,15 +13,24 @@
 from node import Node
 from method import Method
 from function import Function
+from ctype import Type
+from templated import Templated
+
 from cldoc.clang import cindex
 
-class FunctionTemplate(Function):
+class FunctionTemplate(Templated, Function):
     kind = None
 
-class MethodTemplate(Method):
+    def __init__(self, cursor, comment):
+        super(FunctionTemplate, self).__init__(cursor, comment)
+
+class MethodTemplate(Templated, Method):
     kind = None
 
-class FunctionPlexer(Node):
+    def __init__(self, cursor, comment):
+        super(MethodTemplate, self).__init__(cursor, comment)
+
+class FunctionTemplatePlexer(Node):
     kind = cindex.CursorKind.FUNCTION_TEMPLATE
 
     def __new__(cls, cursor, comment):
