@@ -20,7 +20,7 @@ class Method(Function):
     kind = cindex.CursorKind.CXX_METHOD
 
     def __init__(self, cursor, comment):
-        Function.__init__(self, cursor, comment)
+        super(Method, self).__init__(cursor, comment)
 
         self.static = cursor.is_static_method()
         self.virtual = cursor.is_virtual_method()
@@ -79,6 +79,10 @@ class Method(Function):
 
         return cm
 
+    @property
+    def semantic_parent(self):
+        return Node.semantic_parent.fget(self)
+
     def update_abstract(self, cursor):
         if cursor.is_definition() or cursor.get_definition():
             self.abstract = False
