@@ -13,6 +13,9 @@
 from cldoc.clang import cindex
 from cldoc.comment import Comment
 from cldoc.comment import Parser
+
+from cldoc import utf8
+
 import re
 
 class Node(object):
@@ -95,6 +98,45 @@ class Node(object):
     def qid_to(self, qid):
         return self.qid_from_to(qid, self.qid)
 
+    def semantic_path_until(self, parent):
+        ret = []
+
+        if parent == self:
+            return [self]
+
+        p = self
+
+        while (not p is None) and p != parent:
+            ret.insert(0, p)
+
+            sp = p.semantic_parent
+            p = p.parent
+
+            while p != sp:
+                if p == parent:
+                    return ret
+
+                p = p.parent
+
+        return ret
+
+    @property
+    def is_unlabeled(self):
+        return False
+
+    def qlbl_from(self, other):
+        p = other.semantic_path_until(self)
+
+        i = 0
+
+        while i < (len(p) - 1) and p[i].is_unlabeled:
+            i += 1
+
+        return utf8.utf8('::').join(filter(lambda x: x, [q.name for q in p[i:]]))
+
+    def qlbl_to(self, other):
+        return other.qlbl_from(self)
+
     def add_ref(self, cursor):
         self._refs.append(cursor)
         self.add_comment_location(cursor.extent.start)
@@ -105,18 +147,21 @@ class Node(object):
     @property
     def comment_locations(self):
         if self.cursor:
-            yield self.cursor.extent.start
+            ext = self.cursor.extent
+
+            if not ext is None:
+                yield ext.start
 
         for loc in self._comment_locations:
             yield loc
 
     def parse_comment(self):
         # Just extract brief and doc
-        m = Parser.parse(self._comment.text)
+        self._parsed_comment = Parser.parse(self._comment.text)
 
-        if len(m.brief) > 0:
-            self._comment.brief = m.brief
-            self._comment.doc = m.body
+        if len(self._parsed_comment.brief) > 0:
+            self._comment.brief = self._parsed_comment.brief
+            self._comment.doc = self._parsed_comment.body
 
     @property
     def natural_sort_name(self):
@@ -173,14 +218,20 @@ class Node(object):
 
         return ret
 
+    @property
+    def semantic_parent(self):
+        parent = self.parent
+
+        while (not parent is None) and parent.is_anonymous:
+            parent = parent.parent
+
+        return parent
+
     @property
     def qid(self):
         meid = self.name
 
-        parent = self.parent
-
-        while parent and parent.is_anonymous:
-            parent = parent.parent
+        parent = self.semantic_parent
 
         if not parent:
             return meid
