@@ -22,7 +22,8 @@ import nodes
 import includepaths
 import documentmerger
 
-from cldoc import example
+from . import example
+from . import utf8
 
 import os, sys, sets, re, glob, platform
 
@@ -191,7 +192,7 @@ class Tree(documentmerger.DocumentMerger):
 
     def markup_code(self, index):
         for node in self.all_nodes:
-            if not node.comment:
+            if node.comment is None:
                 continue
 
             if not node.comment.doc:
@@ -256,7 +257,7 @@ class Tree(documentmerger.DocumentMerger):
                 comps[i] = ex
 
     def match_ref(self, child, name):
-        if isinstance(name, basestring):
+        if isinstance(name, utf8.string):
             return name == child.name
         else:
             return name.match(child.name)
@@ -280,7 +281,7 @@ class Tree(documentmerger.DocumentMerger):
         if node is None:
             node = self.root
 
-        if node.comment:
+        if not node.comment is None:
             node.comment.resolve_refs(self.find_ref, node)
 
         for child in node.children:
@@ -296,12 +297,25 @@ class Tree(documentmerger.DocumentMerger):
 
         return None
 
+    def c_function_is_constructor(self, node):
+        hints = ['new', 'init', 'alloc', 'create']
+
+        for hint in hints:
+            if node.name.startswith(hint + "_") or \
+               node.name.endswith("_" + hint):
+                return True
+
+        return False
+
     def node_on_c_struct(self, node):
         if isinstance(node, nodes.Method) or \
            not isinstance(node, nodes.Function):
             return None
 
-        decl = self.decl_on_c_struct(node, node.return_type)
+        decl = None
+
+        if self.c_function_is_constructor(node):
+            decl = self.decl_on_c_struct(node, node.return_type)
 
         if not decl:
             args = node.arguments
@@ -434,7 +448,7 @@ class Tree(documentmerger.DocumentMerger):
                         for node in ret:
                             self.register_node(node, par)
 
-                ignoretop = [cindex.CursorKind.TYPE_REF]
+                ignoretop = [cindex.CursorKind.TYPE_REF, cindex.CursorKind.PARM_DECL]
 
                 if (not par or ret is None) and not item.kind in ignoretop:
                     sys.stderr.write("Unhandled cursor: %s\n" % (item.kind))
